@@ -10,6 +10,8 @@
 from ._list import (plist, tlist)
 from ._set  import (pset,  tset)
 from ._dict import (pdict, tdict)
+from ._lazy import (lazy, llist, ldict)
+
 # We don't include the abc types in the __all__; they are probably not as
 # frequently used and don't really need to be here. One can always `import
 # pcollections.abc` if they are needed.
@@ -24,7 +26,8 @@ from ._dict import (pdict, tdict)
 __all__ = [
     "plist", "tlist",
     "pset",  "tset",
-    "pdict", "tdict"
+    "pdict", "tdict",
+    "lazy", "llist", "ldict"
 ]
 
 __version__ = "0.1.0"
